#!/usr/bin/env python

import pygame
import entities
from entities import Kid, ThingsEmitter
import text
import sprite
import images

FPS = 10

DIRECTION_RIGHT = False
DIRECTION_LEFT = True

ANIMATION_IDLE = 0
ANIMATION_RUN = 1

class Game(object):
    def __init__(self):
        pygame.init()
        self.screen = pygame.display.set_mode((800,600))
        self.clock = pygame.time.Clock()
        #init events:
        self.input = GameInput()
        #init game stat:
        images.init_images()
        self.current_game_state = self.game_state_start
        self.text_writer = text.PixelFont(images.IMG_FONT)
        self.start_screen_animation = entities.StartScreenAnimation()
        
        
    def main_loop(self):
        while (True):
            self.input.process()
            if self.input.quit: break
            self.current_game_state()
            pygame.display.flip()
            self.clock.tick(FPS)
        # print "bye!"
        pygame.quit()
       
    
    def game_state_start(self):
        self.screen.fill(0x72af46)  #TODO: use dedicated starting screen
        self.screen.blit(images.IMG_BACKGROUND, (0,0))
        self.text_writer.draw_string("THINGS FROM ABOVE", self.screen, 400, 100, text.ALIGN_CENTER)
        self.text_writer.draw_string("a game by", self.screen, 400, 140, text.ALIGN_CENTER)
        self.text_writer.draw_string("spacebugman and macrobe", self.screen, 400, 180, text.ALIGN_CENTER)
        self.text_writer.draw_string("< or > to move", self.screen, 400, 400, text.ALIGN_CENTER)
        self.text_writer.draw_string("^ to quit", self.screen, 400, 440, text.ALIGN_CENTER)
        self.text_writer.draw_string("press < or > to start", self.screen, 400, 480, text.ALIGN_CENTER)
        self.start_screen_animation.tick_and_draw(self.screen)
        if self.input.left or self.input.right:
            self.kid = Kid()
            self.things = ThingsEmitter(self.kid);
            self.current_game_state = self.game_state_running
    
    def game_state_running(self):
        self.kid.tick(move_left=self.input.left, move_right=self.input.right)
        self.things.tick()
        
        self.screen.blit(images.IMG_BACKGROUND, (0,0))
        self.things.draw(self.screen)
        self.kid.draw(self.screen)
        
        level_message = "level:%3i" % self.things.level_number
        self.text_writer.draw_string(level_message, self.screen, 10, 560, text.ALIGN_LEFT)
        score_message = "score:%5i" % self.things.score
        self.text_writer.draw_string(score_message, self.screen, 760, 560, text.ALIGN_RIGHT)
        
        if not self.kid.is_alive:
            self.current_game_state = self.game_state_gameover
    
    def game_state_gameover(self):
        self.screen.fill(0xcc0000) #TODO: use dedicated game over screen
        self.screen.blit(images.IMG_BACKGROUND, (0,0))
        
        self.text_writer.draw_string("game over", self.screen, 400, 200, text.ALIGN_CENTER)
        self.text_writer.draw_string("your final score was %i" % self.things.score, self.screen, 400, 300, text.ALIGN_CENTER)
        self.text_writer.draw_string("you reached level %i" % self.things.level_number, self.screen, 400, 340, text.ALIGN_CENTER)
        self.text_writer.draw_string("press [] to retry", self.screen, 400, 480, text.ALIGN_CENTER)
        
        self.kid.tick(False, False)
        self.kid.draw(self.screen)

        if self.input.confirm:
            self.start_screen_animation.randomize()
            self.current_game_state = self.game_state_start
    


class GameInput(object):
    def __init__(self):
        pygame.event.set_allowed(None)
        pygame.event.set_allowed(pygame.QUIT)
        
    def process(self):
        keys = pygame.key.get_pressed()
        self.quit = self._is_quit_event() or keys[pygame.K_ESCAPE] or (keys[pygame.K_q] and self._is_cmd_pressed())
        self.left = keys[pygame.K_LEFT] or keys[pygame.K_a]
        self.right = keys[pygame.K_RIGHT] or keys[pygame.K_d]
        self.confirm = keys[pygame.K_SPACE]
    
    def _is_quit_event(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return True
        return False
    
    def _is_cmd_pressed(self):
        return (pygame.KMOD_LMETA | pygame.KMOD_RMETA) & pygame.key.get_mods()
        
if __name__ == '__main__':
    Game().main_loop()